@@ -1,19 +1,25 @@
 import io
 import random
 import re
+import smtplib
 import string
 import sys
 import datetime
+from email.mime.application import MIMEApplication
+from email.mime.multipart import MIMEMultipart
+from email.mime.text import MIMEText
+
 import requests
 from flask import (
-    send_file)
+    send_file, request)
 
 import uuid
 import os
-from pathlib import Path as _PathlibPath
+from pathlib import Path as _PathlibPath, Path
 import base64
 import flask_dictabase
 import hashlib
+from werkzeug.utils import secure_filename
 
 
 def StripNonHex(string):
@@ -192,6 +198,11 @@ def ModIndexLoop(num, min_, max_):
 global app
 
 
+def Setup(a):
+    global app
+    app = a
+
+
 def ListOfDictToJS(l):
     '''
     take in a list of dict
@@ -272,7 +283,7 @@ def PathString(path):
             # when using pipenv
             projPath = _PathlibPath(PROJECT_PATH)
 
-            if path.startswith('/'):
+            if str(path).startswith('/'):
                 if path.startswith(str(projPath)):
                     # path already starts with project path
                     ret = path
@@ -296,18 +307,23 @@ class File:
 
 
 class FormFile(File):
-    def __init__(self, form, key):
-        self._form = form
-        self._key = key
-        super().__init__(form, key)
+    def __init__(self, requestFilesKey):
+        self._key = requestFilesKey
+        self._name = secure_filename(
+            str(uuid.uuid4()) + request.files[self._key].filename  # adds uuid prefix to prevent colisions
+        )
+        super().__init__(self._key)
 
     def SaveTo(self, newPath):
-        self._form[self._key].data.save(PathString(newPath))
+        if _PathlibPath(newPath).is_dir():
+            newPath = _PathlibPath(newPath) / self._name
+
+        request.files[self._key].save(PathString(newPath))
         return SystemFile(newPath)
 
     @property
     def Size(self, asString=False):
-        size = len(self._form[self._key].data)
+        size = len(request.files[self._key])
         if asString:
             sizeString = '{:,} Bytes'.format(size)
             return sizeString
@@ -316,15 +332,18 @@ class FormFile(File):
 
     @property
     def Extension(self):
-        return self._form[self._key].data.filename.split('.')[-1].lower()
+        return request.files[self._key].filename.split('.')[-1].lower()
 
     def Read(self):
-        return self._form[self._key].data.read()
+        return request.files[self._key].read()
+
+    def Data(self):
+        return request.files[self._key]
 
     @property
     def Name(self):
         # returns filename like "image.jpg"
-        return self._form[self._key].data.filename
+        return self._name
 
     def RenderResponse(self):
         return send_file(
@@ -397,6 +416,8 @@ class SystemFile(File):
             'png': 'image',
             'jpeg': 'image',
             'gif': 'image',
+            'jfif': 'image',
+            'ico': 'image',
 
             'flv': 'video',
             'mov': 'video',
@@ -604,3 +625,99 @@ class HashableDict(dict):
             retD[key] = value
 
         return HashableDict(retD)
+
+
+def GetClientIP(raiseForLocalAddress=True):
+    ret = request.remote_addr
+    if raiseForLocalAddress and '127.0.0' in ret:
+        raise TypeError(
+            'reqeust["remote_addr"] "{}" looks like a local address, '
+            'you may need to do "from werkzeug.middleware import proxy_fix; '
+            'app.wsgi_app = proxy_fix.ProxyFix(app.wsgi_app)". '
+            'Or pass GetClientIP(raiseForLocalAddress=False).'
+            'You will also need to add this to your nginx config'
+            '# the below will pass the client IP to flask'
+            'proxy_set_header X-Forwarded-Proto https;'
+            'proxy_set_header X-Forwarded-Host {2};'
+            'proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;'
+        )
+    else:
+        return ret
+
+
+def SendEmail_SMTP(smtpServerURL, smtpUsername, smtpPassword, to, frm, cc=None, bcc=None, subject=None, body=None,
+                   html=None, attachments=None):
+    """
+
+    :param smtpServerURL:
+    :param smtpUsername:
+    :param smtpPassword:
+    :param to:
+    :param frm:
+    :param cc:
+    :param bcc:
+    :param subject:
+    :param body:
+    :param html: body using html, not supported by all email clients
+    :param attachments: list of filepaths
+    :return:
+    """
+
+    # Create message container - the correct MIME type is multipart/alternative.
+    multipart = MIMEMultipart('alternative')
+
+    subject = subject or 'No Subject'
+
+    multipart['Subject'] = subject
+    multipart['From'] = frm
+    multipart['To'] = to
+    if cc:
+        multipart['Cc'] = cc
+    if bcc:
+        multipart['Bcc'] = bcc
+
+    # Create the body of the message (a plain-text and an HTML version).
+
+    # Record the MIME types of both parts - text/plain and text/html.
+    partBody = MIMEText(body, 'plain')
+
+    html = html or f'''
+            <html>
+                <body>
+                    {body}
+                </body>
+            </html>
+            '''
+    partHTML = MIMEText(html, 'html')
+
+    # Attach parts into message container.
+    # According to RFC 2046, the last part of a multipart message, in this case
+    # the HTML message, is best and preferred.
+    multipart.attach(partBody)
+    multipart.attach(partHTML)
+
+    # attachment files
+    attachments = attachments or []
+    for attachment in attachments:
+        with open(attachment, mode='rb') as aFile:
+            file = MIMEApplication(aFile.read())
+            name = Path(attachment).name
+            file.add_header('Content-Disposition', 'attachment', filename=name)
+            multipart.attach(file)
+
+    # send the message
+    with smtplib.SMTP_SSL(
+            host=smtpServerURL,
+            port=465,
+    ) as smtp:
+        smtp.login(smtpUsername, smtpPassword)
+
+        if sys.platform.startswith('win'):
+            smtp.set_debuglevel(1)
+
+        smtp.sendmail(
+            from_addr=frm,
+            to_addrs=to,
+            msg=multipart.as_string(),
+        )
+        smtp.quit()
